@@ -7,6 +7,8 @@ from nctoolkit.runthis import run_this, run_cdo, tidy_command
 from nctoolkit.session import nc_safe, session_info, append_safe, remove_safe
 from nctoolkit.show import nc_variables
 from nctoolkit.temp_file import temp_file
+from nctoolkit.utils import cdo_version
+
 
 
 def arithall(self, stat="divc", x=None):
@@ -23,18 +25,10 @@ def arithall(self, stat="divc", x=None):
 
 def operation(self, method="mul", ff=None, var=None):
     """
-    Method to add, subtract etc. a netcdf file from another one
+    Method to add, subtract etc. a netCDF file from another one
     This is used by add etc.
     """
 
-    # get the cdo version
-    cdo_check = subprocess.run(
-        "cdo --version", shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE
-    )
-    cdo_check = str(cdo_check.stderr).replace("\\n", "")
-    cdo_check = cdo_check.replace("b'", "").strip()
-    cdo_version = cdo_check.split("(")[0].strip().split(" ")[-1]
-
     # If the dataset has to be merged,
     # then this operation will not work without running it first
     if self._merged:
@@ -54,7 +48,7 @@ def operation(self, method="mul", ff=None, var=None):
 
     # make sure the ff file is not removed from safe list in subsequent
     # actions prior to running
-    if (ff is not None) and (session_info["lazy"]):
+    if (ff is not None) and (session_info["lazy"]) and (cdo_version() != "1.9.3"):
         append_safe(ff)
         self._safe.append(ff)
 
@@ -134,11 +128,11 @@ def operation(self, method="mul", ff=None, var=None):
 def multiply(self, x=None, var=None):
     """
     Multiply a dataset
-    This will multiply a dataset by a constant, another dataset or a NetCDF file.
+    This will multiply a dataset by a constant, another dataset or a netCDF file.
     Parameters
     ------------
-    x: int, float, DataSet or netcdf file
-        An int, float, single file dataset or netcdf file to multiply the dataset by.
+    x: int, float, DataSet or netCDF file
+        An int, float, single file dataset or netCDF file to multiply the dataset by.
         If multiplying by a dataset or single file there must only be a single variable
         in it, unless var is supplied. The grids must be the same.
     var: str
@@ -158,7 +152,7 @@ def multiply(self, x=None, var=None):
     Grids in the datasets must match. Multiplication will occur in matching timesteps in data1 and data2. If there is only 1 timestep in data2, then
     the data from that timestep in data2 will multiply the data in all timesteps in data1.
 
-    Multiplying a dataset by the data from another NetCDF file will work in the same way:
+    Multiplying a dataset by the data from another netCDF file will work in the same way:
 
     >>> data1.multiply("example.nc")
     """
@@ -168,8 +162,8 @@ def multiply(self, x=None, var=None):
         arithall(self, stat="mulc", x=x)
         return None
 
-    # 2: dataset or netcdf file multiplication
-    # get the netcdf file(s)
+    # 2: dataset or netCDF file multiplication
+    # get the netCDF file(s)
     if "api.DataSet" in str(type(x)):
         x.run()
         if len(x) == 1:
@@ -188,12 +182,12 @@ def multiply(self, x=None, var=None):
 def subtract(self, x=None, var=None):
     """
     Subtract from a dataset
-    This will subtract a constant, another dataset or a NetCDF file from the dataset.
+    This will subtract a constant, another dataset or a netCDF file from the dataset.
     Parameters
     ------------
-    x: int, float, DataSet or netcdf file
-        An int, float, single file dataset or netcdf file to subtract from the dataset.
-        If a dataset or netcdf is supplied this must only have one variable,
+    x: int, float, DataSet or netCDF file
+        An int, float, single file dataset or netCDF file to subtract from the dataset.
+        If a dataset or netCDF is supplied this must only have one variable,
         unless var is provided. The grids must be the same.
     var: str
         A variable in the x to use for the operation
@@ -212,7 +206,7 @@ def subtract(self, x=None, var=None):
     Grids in the datasets must match. Division will occur in matching timesteps in data1 and data2 if there are matching timesteps. If there is only 1 timestep in data2, then
     the data from that timestep in data2 will be subtracted from the data in all timesteps in data1.
 
-    Subtracting of the data from another NetCDF file will work in the same way:
+    Subtracting of the data from another netCDF file will work in the same way:
 
     >>> data1.subtract("example.nc")
     """
@@ -222,8 +216,8 @@ def subtract(self, x=None, var=None):
         arithall(self, stat="subc", x=x)
         return None
 
-    # 2: dataset or netcdf file subtraction
-    # get the netcdf file(s)
+    # 2: dataset or netCDF file subtraction
+    # get the netCDF file(s)
     if "api.DataSet" in str(type(x)):
         x.run()
         if len(x) == 1:
@@ -242,12 +236,12 @@ def subtract(self, x=None, var=None):
 def add(self, x=None, var=None):
     """
     Add to a dataset
-    This will add a constant, another dataset or a NetCDF file to the dataset.
+    This will add a constant, another dataset or a netCDF file to the dataset.
     Parameters
     ------------
-    x: int, float, DataSet or netcdf file
-        An int, float, single file dataset or netcdf file to add to the dataset.
-        If a dataset or netcdf file is supplied, this must have only one variable,
+    x: int, float, DataSet or netCDF file
+        An int, float, single file dataset or netCDF file to add to the dataset.
+        If a dataset or netCDF file is supplied, this must have only one variable,
         unless var is provided. The grids must be the same.
     var: str
         A variable in the x to use for the operation
@@ -266,21 +260,20 @@ def add(self, x=None, var=None):
     Grids in the datasets must match. Addition will occur in matching timesteps in data1 and data2. If there is only 1 timestep in data2, then
     the data from that timestep will be added to the data in all data1 time steps.
 
-    Adding the data from another NetCDF file will work in the same way:
+    Adding the data from another netCDF file will work in the same way:
 
     >>> data1.add("example.nc")
 
 
     """
 
-
     # 1: int, float addition
     if isinstance(x, (int, float)):
         arithall(self, stat="addc", x=x)
         return None
 
-    # 2: dataset or netcdf file addition
-    # get the netcdf file(s)
+    # 2: dataset or netCDF file addition
+    # get the netCDF file(s)
     if "api.DataSet" in str(type(x)):
         x.run()
         if len(x) == 1:
@@ -299,12 +292,12 @@ def add(self, x=None, var=None):
 def divide(self, x=None, var=None):
     """
     Divide the data
-    This will divide the dataset by a constant, another dataset or a NetCDF file.
+    This will divide the dataset by a constant, another dataset or a netCDF file.
     Parameters
     ------------
-    x: int, float, DataSet or netcdf file
-        An int, float, single file dataset or netcdf file to divide the dataset by.
-        If a dataset or netcdf file is supplied, this must have only one variable,
+    x: int, float, DataSet or netCDF file
+        An int, float, single file dataset or netCDF file to divide the dataset by.
+        If a dataset or netCDF file is supplied, this must have only one variable,
         unless var is provided. The grids must be the same.
     var: str
         A variable in the x to use for the operation
@@ -323,7 +316,7 @@ def divide(self, x=None, var=None):
     Grids in the datasets must match. Division will occur in matching timesteps in data1 and data2. If there is only 1 timestep in data2, then
     the data from that timeste in data2 will divided the data in all data1 time steps.
 
-    Adding the data from another NetCDF file will work in the same way:
+    Adding the data from another netCDF file will work in the same way:
 
     >>> data1.divide("example.nc")
     """
@@ -333,8 +326,8 @@ def divide(self, x=None, var=None):
         arithall(self, stat="divc", x=x)
         return None
 
-    # 2: dataset or netcdf file division
-    # get the netcdf file(s)
+    # 2: dataset or netCDF file division
+    # get the netCDF file(s)
     if "api.DataSet" in str(type(x)):
         x.run()
         if len(x) == 1:
