@@ -18,8 +18,17 @@ from netCDF4 import Dataset
 from nctoolkit.cleanup import cleanup, clean_all, temp_check
 from nctoolkit.flatten import str_flatten
 from nctoolkit.runthis import run_cdo
-from nctoolkit.session import nc_protected, session_info, nc_safe, append_safe, remove_safe, append_protected, remove_protected, append_tempdirs
-from nctoolkit.session import nc_safe_par
+from nctoolkit.session import (
+    nc_protected,
+    session_info,
+    nc_safe,
+    append_safe,
+    remove_safe,
+    append_protected,
+    remove_protected,
+    append_tempdirs,
+)
+from nctoolkit.session import nc_safe_par, temp_dirs, nc_protected
 from nctoolkit.show import (
     nc_variables,
     nc_years,
@@ -79,7 +88,6 @@ session_info["parallel"] = False
 session_info["interactive"] = sys.__stdin__.isatty()
 
 
-
 if platform.system() == "Linux":
     result = os.statvfs("/tmp/")
     session_info["size"] = result.f_frsize * result.f_bavail
@@ -128,14 +136,7 @@ def options(**kwargs):
 
     """
 
-    valid_keys = [
-        "thread_safe",
-        "lazy",
-        "cores",
-        "precision",
-        "temp_dir",
-        "parallel"
-    ]
+    valid_keys = ["thread_safe", "lazy", "cores", "precision", "temp_dir", "parallel"]
 
     for key in kwargs:
         if key not in valid_keys:
@@ -145,19 +146,6 @@ def options(**kwargs):
             if type(kwargs[key]) is not bool:
                 raise TypeError(f"{key} should be boolean")
 
-            if kwargs[key]:
-                if len(nc_safe) > 0:
-                    for ff in nc_safe:
-                        nc_safe_par.append(ff)
-                        nc_safe.remove(ff)
-
-            if kwargs[key] == False:
-                if len(nc_safe_par) > 0:
-                    for ff in nc_safe_par:
-                        nc_safe.append(ff)
-                        nc_safe_par.remove(ff)
-
-
         if type(kwargs[key]) is not bool:
             if key == "temp_dir":
                 if type(kwargs[key]) is str:
@@ -189,6 +177,39 @@ def options(**kwargs):
         else:
             session_info[key] = kwargs[key]
 
+            # update safe-lists etc. if running in parallel
+            if kwargs[key] and key == "parallel":
+
+                if len(temp_dirs) > 0:
+                    for ff in temp_dirs:
+                        append_tempdirs(ff)
+
+                if len(nc_safe) > 0:
+                    for ff in nc_safe:
+                        nc_safe_par.append(ff)
+
+                if len(nc_protected) > 0:
+                    for ff in nc_protected:
+                        nc_protected_par.append(ff)
+                        nc_protected.remove(ff)
+                        nc_protected.remove(ff)
+
+            if (kwargs[key] == False) and key == "parallel":
+
+                if len(temp_dirs_par) > 0:
+                    for ff in temp_dirs_par:
+                        append_tempdirs(ff)
+
+                if len(nc_safe_par) > 0:
+                    for ff in nc_safe_par:
+                        nc_safe.append(ff)
+                        nc_safe_par.remove(ff)
+
+                if len(nc_protected_par) > 0:
+                    for ff in nc_protected_par:
+                        nc_protected.append(ff)
+                        nc_protected_par.remove(ff)
+
 
 # if nctoolkitrc exists, we need to read the possible options from there...
 
@@ -201,6 +222,7 @@ def find_config():
 
     # now look in the home directory....
     from os.path import expanduser
+
     home = expanduser("~")
     for ff in [".nctoolkitrc", "nctoolkitrc"]:
         if os.path.exists(home + "/" + ff):
@@ -212,13 +234,7 @@ def find_config():
 config_file = find_config()
 
 if config_file is not None:
-    valid_keys = [
-        "thread_safe",
-        "lazy",
-        "cores",
-        "precision",
-        "temp_dir"
-    ]
+    valid_keys = ["thread_safe", "lazy", "cores", "precision", "temp_dir"]
 
     file1 = open(config_file, "r")
     Lines = file1.readlines()
@@ -333,65 +349,6 @@ def is_url(x):
     return re.match(regex, x) is not None
 
 
-#def options(**kwargs):
-#    """
-#    Define session options.
-#    Set the options in the session. Available options are thread_safe and lazy.
-#    Set thread_safe = True if hdf5 was built to be thread safe.
-#    Set lazy = True if you want methods to evaluate lazy by default.
-#
-#    Parameters
-#    ---------------
-#    **kwargs
-#        Define options using key, value pairs.
-#
-#    """
-#
-#    valid_keys = [
-#        "thread_safe",
-#        "lazy",
-#        "cores",
-#        "precision",
-#        "user",
-#        "password",
-#        "temp_dir",
-#    ]
-#
-#    for key in kwargs:
-#        if key not in valid_keys:
-#            raise AttributeError(key + " is not a valid option")
-#        if type(kwargs[key]) is not bool:
-#            if key == "temp_dir":
-#                if type(kwargs[key]) is str:
-#                    if os.path.exists(kwargs[key]) == False:
-#                        raise ValueError("The temp_dir specified does not exist!")
-#                    session_info[key] = os.path.abspath(kwargs[key])
-#                    session_info["user_dir"] = True
-#                return None
-#
-#            if key == "cores":
-#                if type(kwargs[key]) is int:
-#                    if kwargs[key] > mp.cpu_count():
-#                        raise ValueError(
-#                            str(kwargs[key])
-#                            + " is greater than the number of system cores ("
-#                            + str(mp.cpu_count())
-#                            + ")"
-#                        )
-#                    session_info[key] = kwargs[key]
-#                else:
-#                    raise TypeError("cores must be an int")
-#            else:
-#                if key == "precision":
-#                    if kwargs[key] not in ["I8", "I16", "I32", "F32", "F64"]:
-#                        raise ValueError("precision supplied is not valid!")
-#                    session_info[key] = kwargs[key]
-#                else:
-#                    raise AttributeError(key + " is not valid session info!")
-#        else:
-#            session_info[key] = kwargs[key]
-#
-
 def convert_bytes(num):
     """
     A function to make file size human readable
@@ -411,22 +368,27 @@ def file_size(file_path):
         return file_info.st_size
 
 
-def open_data(x=[], suppress_messages=False, checks=False, **kwargs):
+def open_data(x=[], checks=False, **kwargs):
     """
-    Read netcdf data as a DataSet object
+    Read netCDF data as a DataSet object
 
     Parameters
     ---------------
     x : str or list
-        A string or list of netcdf files or a single url. The function will check the
+        A string or list of netCDF files or a single url. The function will check the
         files exist. If x is not a list, but an iterable it will be converted to a list.
         If a *.nc style wildcard is supplied, open_data will use all files available.
         By default an empty dataset is created, ie. using open_data() will create an empty
         dataset that can then be expanded using append.
-    thredds : boolean
-        Are you accessing a thredds server? Must end with .nc.
     checks: boolean
         Do you want basic checks to ensure cdo can read files?
+    **kwargs: kwargs
+        Optional arguments for internal use by open_thredds and open_url.
+
+    Returns
+    ---------------
+    open_data : nctoolkit.DataSet
+
 
     Examples
     ------------
@@ -436,23 +398,16 @@ def open_data(x=[], suppress_messages=False, checks=False, **kwargs):
     >>> import nctoolkit as nc
     >>> data = nc.open_data("example.nc")
 
-
     If you want to open a list of files as a multi-file dataset, you would do something like this:
 
-
     >>> import nctoolkit as nc
     >>> data = nc.open_data(["file1.nc", "file2.nc", "file3.nc"])
 
-
     If you wanted to open all files in a directory "data" as a multi-file dataset, you can use a wildcard:
 
     >>> import nctoolkit as nc
     >>> data = nc.open_data("data/*.nc")
 
-
-
-
-
     """
     # from nctoolkit.temp_file import temp_file
 
@@ -606,11 +561,6 @@ def open_data(x=[], suppress_messages=False, checks=False, **kwargs):
 
     if type(x) is list:
         if thredds is False:
-            if checks:
-                if suppress_messages is False:
-                    warnings.warn("Performing basic checks on ensemble files")
-            #if len(x) == 0:
-            #    raise ValueError("You have not provided any files!")
 
             if len(x) > 1:
                 for ff in x:
@@ -671,6 +621,10 @@ def open_thredds(x=None, wait=None, checks=False):
     wait : int
         Time to wait for thredds server to be checked. Limitless if not supplied.
 
+    Returns
+    ---------------
+    open_thredds : nctoolkit.DataSet
+
     Examples
     ------------
 
@@ -702,7 +656,7 @@ def open_thredds(x=None, wait=None, checks=False):
 
 def open_url(x=None, ftp_details=None, wait=None, file_stop=None):
     """
-    Read netcdf data from a url as a DataSet object
+    Read netCDF data from a url as a DataSet object
 
     Parameters
     ---------------
@@ -716,6 +670,10 @@ def open_url(x=None, ftp_details=None, wait=None, file_stop=None):
     file_stop : int
         Time limit, in minutes, for individual attempts at downloading data. This is useful to get around download freezes.
 
+    Returns
+    ---------------
+    open_url : nctoolkit.DataSet
+
     Examples
     ------------
 
@@ -823,7 +781,12 @@ def cor_time(x=None, y=None):
 
     if len(x.variables) == 1:
         command = (
-            "cdo -setname,cor -timcor " + a.current[0] + " " + b.current[0] + " " + target
+            "cdo -setname,cor -timcor "
+            + a.current[0]
+            + " "
+            + b.current[0]
+            + " "
+            + target
         )
     else:
         command = "cdo -timcor " + a.current[0] + " " + b.current[0] + " " + target
@@ -879,7 +842,12 @@ def cor_space(x=None, y=None):
 
     if len(x.variables) == 1:
         command = (
-            "cdo -setname,cor -fldcor " + a.current[0] + " " + b.current[0] + " " + target
+            "cdo -setname,cor -fldcor "
+            + a.current[0]
+            + " "
+            + b.current[0]
+            + " "
+            + target
         )
     else:
         command = "cdo -fldcor " + a.current[0] + " " + b.current[0] + " " + target
@@ -895,7 +863,7 @@ def cor_space(x=None, y=None):
 
 class DataSet(object):
     """
-    A modifiable ensemble of netcdf files
+    A modifiable ensemble of netCDF files
     """
 
     def __init__(self, start=""):
@@ -920,9 +888,13 @@ class DataSet(object):
         self._thredds = False
         self._zip = False
         self._format = None
+
+        self._grid = None
+        self._weights = None
         # track number of held over commands
         self._ncommands = 0
 
+
     def __getitem__(self, index):
 
         return self.current[index]
@@ -935,7 +907,6 @@ class DataSet(object):
             yield ff
         return
 
-
     def __repr__(self):
         current = str(len(self)) + " member ensemble"
 
@@ -985,12 +956,7 @@ class DataSet(object):
         result = "Number of files in ensemble: " + str(len(self)) + "\n"
         result = result + "Ensemble size: " + sum_size + "\n"
         result = (
-            result
-            + "Smallest file: "
-            + smallest_file
-            + " has size "
-            + min_size
-            + "\n"
+            result + "Smallest file: " + smallest_file + " has size " + min_size + "\n"
         )
         result = result + "Largest file: " + largest_file + " has size " + max_size
         return result
@@ -1098,7 +1064,7 @@ class DataSet(object):
         This will only display the variables in the first file of an ensemble.
         """
 
-        if len(self) >1:
+        if len(self) > 1:
             return (
                 "This DataSet object is a mult-file dataset. Please inspect individual"
                 "files using nc_variables"
@@ -1233,6 +1199,10 @@ class DataSet(object):
             remove_safe(ff)
         for ff in self._safe:
             remove_safe(ff)
+        if self._weights is not None:
+            remove_safe(self._weights)
+        if self._weights is not None:
+            remove_safe(self._grid)
 
         cleanup()
 
@@ -1264,6 +1234,9 @@ class DataSet(object):
 
     from nctoolkit.compare import compare
 
+    from nctoolkit.compare_data import gt
+    from nctoolkit.compare_data import lt
+
     from nctoolkit.corr import cor_space
     from nctoolkit.corr import cor_time
 
@@ -1278,9 +1251,7 @@ class DataSet(object):
     from nctoolkit.ensembles import ensemble_sum
 
     from nctoolkit.ensembles import ensemble_percentile
-    from nctoolkit.expr import mutate
-    from nctoolkit.expr import transmute
-    from nctoolkit.expr import sum_all
+    from nctoolkit.esoteric import assign_coords
 
     from nctoolkit.fldstat import spatial_mean
     from nctoolkit.fldstat import spatial_min
@@ -1318,7 +1289,6 @@ class DataSet(object):
 
     from nctoolkit.regrid import regrid
 
-    from nctoolkit.release import run
 
     from nctoolkit.rename import rename
 
@@ -1330,6 +1300,8 @@ class DataSet(object):
     from nctoolkit.rollstat import rolling_range
     from nctoolkit.rollstat import rolling_sum
 
+    from nctoolkit.run import run
+
     from nctoolkit.select import select
 
     from nctoolkit.setters import set_date
@@ -1337,26 +1309,12 @@ class DataSet(object):
     from nctoolkit.setters import set_units
     from nctoolkit.setters import set_longnames
 
-    from nctoolkit.shift import shift_hours
-    from nctoolkit.shift import shift_days
-    from nctoolkit.shift import shift_months
-    from nctoolkit.shift import shift_years
     from nctoolkit.shift import shift
 
     from nctoolkit.split import split
 
     from nctoolkit.strip_vars import strip_variables
-
-    from nctoolkit.time_stat import mean
-    from nctoolkit.time_stat import percentile
-    from nctoolkit.time_stat import max
-    from nctoolkit.time_stat import median
-    from nctoolkit.time_stat import min
-    from nctoolkit.time_stat import range
-    from nctoolkit.time_stat import variance
-    from nctoolkit.time_stat import stdev
-    from nctoolkit.time_stat import sum
-    from nctoolkit.time_stat import cum_sum
+    from nctoolkit.sumall import sum_all
 
     from nctoolkit.temporal_stat import tmean
     from nctoolkit.temporal_stat import tpercentile
@@ -1369,6 +1327,11 @@ class DataSet(object):
     from nctoolkit.temporal_stat import tsum
     from nctoolkit.temporal_stat import tcumsum
 
+    from nctoolkit.thresholds import first_above
+    from nctoolkit.thresholds import first_below
+    from nctoolkit.thresholds import last_above
+    from nctoolkit.thresholds import last_below
+
     from nctoolkit.to_lonlat import to_latlon
 
     from nctoolkit.to_nc import to_nc
@@ -1388,6 +1351,7 @@ class DataSet(object):
     from nctoolkit.verticals import bottom_mask
     from nctoolkit.verticals import invert_levels
 
+
     from nctoolkit.zip import zip
 
     from nctoolkit.zonals import zonal_mean
@@ -1397,46 +1361,9 @@ class DataSet(object):
 
     # deprecated stuff
 
-    from nctoolkit.deprecated import var
-    from nctoolkit.deprecated import cum_sum
-    from nctoolkit.deprecated import vertical_cum_sum
-    from nctoolkit.deprecated import view
+    from nctoolkit.deprecated import mutate
+    from nctoolkit.deprecated import transmute
 
     from nctoolkit.deprecated import cell_areas
-
-    from nctoolkit.deprecated import daily_mean
-    from nctoolkit.deprecated import daily_min
-    from nctoolkit.deprecated import daily_max
-    from nctoolkit.deprecated import daily_range
-    from nctoolkit.deprecated import daily_sum
-
-    from nctoolkit.deprecated import daily_mean_climatology
-    from nctoolkit.deprecated import daily_min_climatology
-    from nctoolkit.deprecated import daily_max_climatology
-    from nctoolkit.deprecated import daily_range_climatology
-
-    from nctoolkit.deprecated import monthly_mean
-    from nctoolkit.deprecated import monthly_min
-    from nctoolkit.deprecated import monthly_max
-    from nctoolkit.deprecated import monthly_sum
-    from nctoolkit.deprecated import monthly_range
-    from nctoolkit.deprecated import monthly_mean_climatology
-    from nctoolkit.deprecated import monthly_min_climatology
-    from nctoolkit.deprecated import monthly_max_climatology
-    from nctoolkit.deprecated import monthly_range_climatology
-    from nctoolkit.deprecated import annual_mean
-    from nctoolkit.deprecated import annual_min
-    from nctoolkit.deprecated import annual_max
-    from nctoolkit.deprecated import annual_range
-    from nctoolkit.deprecated import annual_sum
-    from nctoolkit.deprecated import seasonal_mean
-    from nctoolkit.deprecated import seasonal_min
-    from nctoolkit.deprecated import seasonal_max
-    from nctoolkit.deprecated import seasonal_range
-
-    from nctoolkit.deprecated import seasonal_mean_climatology
-    from nctoolkit.deprecated import seasonal_min_climatology
-    from nctoolkit.deprecated import seasonal_max_climatology
-    from nctoolkit.deprecated import seasonal_range_climatology
     from nctoolkit.deprecated import remove_variables
     from nctoolkit.compare import compare_all
