@@ -12,20 +12,38 @@ from nctoolkit.session import nc_safe, append_safe, remove_safe
 from nctoolkit.temp_file import temp_file
 
 
-def regrid(self, grid=None, method="bil"):
+def regrid(self, grid=None, method="bil", recycle = False):
     """
     Regrid a dataset to a target grid
 
     Parameters
     -------------
-    grid : nctoolkit.DataSet, pandas data frame or netcdf file
+    grid : nctoolkit.DataSet, pandas data frame or netCDF file
         The grid to remap to
 
     method : str
-        Remapping method. Defaults to "bil". Methods available are: bilinear - "bil";
-        nearest neighbour - "nn" - "nearest neighbour"; "bic" - "bicubic interpolation".
+        Remapping method. Defaults to "bil". Methods available are:
+        bilinear - "bil";
+        nearest neighbour - "nn" - "nearest neighbour"
+        bicubic interpolation - "bic"
+        Distance-weighted average - "dis"
+        First order conservative remapping - "con"
+        Second order conservative remapping - "con2"
+        Large area fraction remapping - "laf"
     """
 
+    valid_methods = ["bil", "nn", "bic", "dis", "con", "con2", "laf"]
+
+    if "DataSet" in str(type(grid)):
+        if grid._weights is not None and grid._grid is not None:
+            target_grid = grid._grid
+            weights_nc = grid._weights
+            cdo_command = f"cdo -remap,{target_grid},{weights_nc}"
+            run_this(cdo_command, self, output="ensemble")
+            print("recycling")
+
+            return None
+
 
     del_grid = None
     if grid is None:
@@ -45,14 +63,15 @@ def regrid(self, grid=None, method="bil"):
     if "DataSet" in str(type(grid)):
         grid.run()
         grid = grid.current[0]
-        warnings.warn(message="The first file in dataset used for regridding!")
+        if len(grid) > 1:
+            warnings.warn(message="The first file in dataset used for regridding!")
         grid_type = "nc"
 
     if grid_type is None:
         raise ValueError("grid supplied is not valid")
 
     # check that the remapping method is valid
-    if (method in {"bil", "bic", "nn"}) is False:
+    if (method in valid_methods) is False:
         raise ValueError("remapping method is invalid. Please check")
 
     # check the number of grids in the dataset
@@ -63,6 +82,10 @@ def regrid(self, grid=None, method="bil"):
 
     self.run()
 
+    if len(self) > 1 and recycle:
+        raise ValueError("You cannot recycle multi-file datasets")
+
+
     for ff in self:
         cdo_result = subprocess.run(
             f"cdo griddes {ff}",
@@ -86,6 +109,8 @@ def regrid(self, grid=None, method="bil"):
             target_grid = grid
     new_files = []
 
+    if recycle:
+        append_safe(target_grid)
 
     for key in grid_split:
         # first we need to generate the weights for remapping
@@ -96,6 +121,7 @@ def regrid(self, grid=None, method="bil"):
 
         weights_nc = temp_file("nc")
 
+
         cdo_command = (
             f"cdo -gen{method},{target_grid} {tracker.current[0]} {weights_nc}"
         )
@@ -107,31 +133,31 @@ def regrid(self, grid=None, method="bil"):
             remove_safe(weights_nc)
             raise ValueError(e)
 
+        if recycle:
+            self._weights = weights_nc
+            self._grid = target_grid
+
         cdo_command = f"cdo -remap,{target_grid},{weights_nc}"
 
         tracker._execute = True
 
         run_this(cdo_command, tracker, output="ensemble")
 
-        remove_safe(weights_nc)
-
+        if recycle == False:
+            remove_safe(weights_nc)
 
         new_files += tracker.current
 
-
         self.history += tracker.history
 
         self._hold_history = copy.deepcopy(self.history)
 
-
-
     if del_grid is not None:
-        remove_safe(del_grid)
-
+        if recycle == False:
+            remove_safe(del_grid)
 
     self.current = new_files
 
-
     self._thredds = False
     cleanup()
     self.disk_clean()
