@@ -23,7 +23,7 @@ def to_nc(self, out, zip=True, overwrite=False):
 
     Examples
     ------------
-    If you want to export a dataset to a NetCDF file, do the following:
+    If you want to export a dataset to a netCDF file, do the following:
 
         >>> data.to_nc("out.nc")
 
@@ -38,6 +38,11 @@ def to_nc(self, out, zip=True, overwrite=False):
 
     """
 
+    out_dir = os.path.dirname(out)
+
+    if os.path.exists(out_dir) == False:
+        raise ValueError(f"{out_dir} does not exist!")
+
     # If you are trying to overwrite a file in self.current, cdo cannot simultaneously have it opened and written to
     if out in self and (overwrite is True):
         self.run()
