@@ -14,6 +14,10 @@ def to_xarray(self, decode_times=True, cdo_times=False):
     cdo_times: boolean
         Set to True if you do not want CDO to decode the times
 
+    Returns
+    ---------------
+    to_xarray :  xarray.Dataset
+
     Examples
     ------------
     If you want to convert a dataset to an xarray dataset, do the following:
@@ -98,6 +102,10 @@ def to_dataframe(self, decode_times=True, cdo_times=False):
     cdo_times: boolean
         Set to True if you do not want CDO to decode the times
 
+    Returns
+    ---------------
+    to_dataframe :  pandas.DataFrame
+
     """
     # everything must be run first
     self.run()
